#!/usr/bin/python3

# This amulet code is to test the lamp charm.

import amulet
import requests

# The number of seconds to wait for Juju to set up the environment.
seconds = 900
# The lamp configuration to test.
lamp_configuration = {
    'website-database': 'lampdatabase',
    'database-user': 'lamp-user',
    'website-bzr': 'lp:~mbruzek/charms/precise/lamp-tests/trunk',
    'bzr-update': 'yes'
}
# The mysql charm needs a configuration setting to deploy correctly.
mysql_configuration = {'dataset-size': '512M'}

d = amulet.Deployment(series='trusty')
# Add the lamp charm to the deployment.
d.add('lamp')
# Add the mysql charm to the deployment.
d.add('mysql')
# Configure the lamp charm.
d.configure('lamp', lamp_configuration)
# Configure the mysql charm so it deploys correctly.
d.configure('mysql', mysql_configuration)
# Relate the lamp and mysql charms.
d.relate('lamp:shared-db', 'mysql:shared-db')
# Expose the open ports on lamp.
d.expose('lamp')

# Deploy the environment and wait for it to setup.
try:
    d.setup(timeout=seconds)
    d.sentry.wait(seconds)
except amulet.helpers.TimeoutError:
    message = 'The environment did not setup in %d seconds.' % seconds
    # The SKIP status enables skip or fail the test based on configuration.
    amulet.raise_status(amulet.SKIP, msg=message)
except:
    raise

# Get the sentry unit for mysql.
mysql_unit = d.sentry.unit['mysql/0']

# Get the sentry unit for lamp.
lamp_unit = d.sentry.unit['lamp/0']

# Get the public address for the system running the lamp charm.
lamp_address = lamp_unit.info['public-address']

###############################################################################
## Verify Linux (the L in LAMP)
###############################################################################
command = 'uname -a'
print(command)
# Run the command to get the Linux kernel version.
output, code = lamp_unit.run(command)
print(output)
if code != 0 and output.find('Linux') != -1:
    message = 'Unable to get the Linux kernel version:\n%s' % output
    amulet.raise_status(amulet.FAIL, msg=message)

###############################################################################
## Verify Apache (the A in LAMP)
###############################################################################
command = 'sudo service apache2 status'
print(command)
# Run the command to see if apache2 is running.
output, code = lamp_unit.run(command)
print(output)
if code != 0:
    message = 'Apache2 does not seem to be running on %s' % lamp_address
    print(message)
    amulet.raise_status(amulet.FAIL, msg=message)

###############################################################################
## Verify MySQL (The M in LAMP)
###############################################################################
# Verify that lamp was related to mysql.
lamp_relation = lamp_unit.relation('shared-db', 'mysql:shared-db')
print('lamp relation to mysql')
for key, value in lamp_relation.items():
    print(key, value)
# Verify that mysql was related to lamp.
mysql_relation = mysql_unit.relation('shared-db', 'lamp:shared-db')
print('mysql relation to lamp')
for key, value in mysql_relation.items():
    print(key, value)
# Get the username from the mysql relation to lamp.
mysql_user = lamp_relation['username']
# Verify the relation has the right database-user value.
if mysql_user != lamp_configuration['database-user']:
    message = 'The database user {0} did not match expected {1}'.format(
              mysql_user, lamp_configuration['database-user'])
    amulet.raise_status(amulet.FAIL, msg=message)
# Get the database from the mysql relation to lamp.
mysql_database = lamp_relation['database']
# Verify the relation has the right database name value.
if mysql_database != lamp_configuration['website-database']:
    message = 'The database name {0} did not match expected {1}'.format(
              mysql_database, lamp_configuration['website-database'])
    amulet.raise_status(amulet.FAIL, msg=message)
# Get the db_host from the lamp relation to mysql.
mysql_ip = mysql_relation['db_host']
# Get the password from the lamp relation to mysql.
mysql_password = mysql_relation['password']
# Create the command to get the mysql status with username and password.
command = 'mysqladmin status -h {0} -u {1} --password={2}'.format(mysql_ip,
          mysql_user, mysql_password)
print(command)
output, code = lamp_unit.run(command)
print(output)
if code != 0:
    message = 'Unable to get the status of mysql server at %s' % mysql_ip
    amulet.raise_status(amulet.FAIL, msg=message)

###############################################################################
## Verify PHP (the P in LAMP)
###############################################################################
# Create a URL string to the lamp server.
lamp_url = 'http://%s' % lamp_address
# The name of the file that calls the phpinfo() method.
phpinfo = 'phpinfo.php'
# Create a url to the phpinfo file on the lamp server.
phpinfo_url = '{0}/{1}'.format(lamp_url, phpinfo)
print(phpinfo_url)
# Get the lamp url with the authentication for guest.
response = requests.get(phpinfo_url)
# Raise an exception if response is not 200 OK.
response.raise_for_status()
# Look for PHP Version in the response.
version_index = response.text.find('PHP Version')
# Look for PHP API in the response.
api_index = response.text.find('PHP API')
found_php_version = version_index != -1
found_php_api = api_index != -1
if found_php_version and found_php_api:
    print(response.text[version_index:].split('\n')[0])
    print(response.text[api_index:].split('\n')[0])
    print('Verified PHP was running on lamp server %s' % lamp_address)
else:
    message = 'PHP was not running correctly on %s' % lamp_address
    print(message)
    print(response.text)
    amulet.raise_status(amulet.FAIL, msg=message)
# The name of the file that connects to the database.
connect_database_php = 'connect_database.php'
# Create a url to the connect_database.php file.
connect_database_php_url = '{0}/{1}'.format(lamp_url, connect_database_php)
print(connect_database_php_url)
# Get the url that returns the connect_database.php results.
response = requests.get(connect_database_php_url)
print(response.text)
# Raise an exception if the response was not 200 OK.
response.raise_for_status()
# Look for the database connected success message in the response.
connect_to_mysql = response.text.find('Connected') != -1
# Look for the datbase selected success message in the response.
selected_database = response.text.find('Selected database') != -1
if connect_to_mysql and selected_database:
    print('Verified the PHP code could connect to the mysql database.')
else:
    message = 'PHP could not connect to the mysql:\n%s' % response.text
    amulet.raise_status(amulet.FAIL, msg=message)

print('The lamp deploy test completed successfully.')
